package contentProvider;

import java.io.BufferedInputStream;
import java.io.File;
import java.io.FileFilter;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.List;

import com.qoppa.spi.ContentProvider;
import com.qoppa.spi.Session;

/**
 * This class is a sample that implements the ContentProvider interface that works with PDF Automation Server to
 * get and save content.
 * 
 * The class defines a "base folder", under which all files should be located.
 * 
 * @author Qoppa Software
 *
 */
public class SampleFileContentProvider implements ContentProvider
{
	private final static String BASE_FOLDER = "/home/qoppa";
	/**
	 * Gets the human readable name of the provider.
	 * @return The human readable name of the provider
	 */
	@Override
	public String getProviderName()
	{
		return "Sample File Content Provider";
	}
	
	/**
	 * Create a new folder a the specified path.  This should also create any parent directories that don't yet exist.
	 * @param folderPath  The path to the new folder.  It is relative to the base directory.
	 * @param authentication The Authorization header from the request.
	 * @param session The existing Session object.
	 * @return True if the folder was create successfully or false if there was a problem creating the new folder or any of its ancestors.
	 */
	@Override
	public boolean createFolder(String folderPath, String authentication, Session session)
	{
		return new File(BASE_FOLDER, folderPath).mkdirs();
	}
	
	/**
	 * Returns true if a file or folder exists at the specified path.
	 * @param path The path to the file or folder relative to the base directory.
	 * @param authentication The Authorization header from the request.
	 * @param session The existing Session object.
	 * @return True if the file or folder exists, false if it doesn't exist.
	 */
	@Override
	public boolean exists(String path, String authentication, Session session)
	{
		return new File(BASE_FOLDER, path).exists();
	}
	
	/**
	 * Gets the size in bytes of file at the specified path.
	 * @param path The path of the file relative to the base directory.
	 * @param authentication The Authorization header from the request.
	 * @param session The existing Session object.
	 * @return The size in bytes of the file at the specified path.
	 */
	@Override
	public long getFileSize(String path, String authentication, Session session)
	{
		return new File(BASE_FOLDER, path).length();
	}
	
	/**
	 * Gets the input stream of the file at the specified path.
	 * @param path The path of the file relative to the base directory.
	 * @param authentication The Authorization header from the request.
	 * @param session The existing Session object.
	 * @return The input stream of the file at the specified path.
	 * @throws IOException
	 */
	@Override
	public InputStream getInputStream(String path, String authentication, Session session) throws IOException
	{
		return new BufferedInputStream(new FileInputStream(new File(BASE_FOLDER, path)));
	}

	/**
	 * Uploads the file to the specified path.
	 * 
	 * @param in The file's input stream.
	 * @param length The length of the input stream.  May be -1 if unknown.
	 * @param path The path of the file relative to the base directory.
	 * @param authentication The Authorization header from the request.
	 * @param session The existing Session object.
	 */
	@Override
	public void upload(InputStream instream, long length, String path, String authentication, Session session)
	{
		try
		{
			OutputStream outstream = new FileOutputStream(new File(BASE_FOLDER, path));
			copyStreams(instream, outstream);
			outstream.close();			
		}
		catch (Exception e)
		{
			e.printStackTrace();
		}
	}
	
	/**
	 * Deletes the file at the specified path.
	 * 
	 * @param path The path of the file relative to the base directory.
	 * @param authentication The Authorization header from the request.
	 * @param session The existing Session object.
	 * @return True if the file was successfully deleted or false if there was a problem deleting the file.
	 */
	@Override
	public boolean deleteFile(String path, String authentication, Session session)
	{
		return new File(BASE_FOLDER, path).delete();
	}
	
	/**
	 * Deletes the folder at the specified path.  If recursive is set to false, the folder should only be deleted if it is empty.
	 * @param folderPath The path of the folder relative to the base directory.
	 * @param recursive Set to true if all sub-folders should be deleted.  
	 * @param authentication The Authorization header from the request.
	 * @param session The existing Session object.
	 * @return True if the folder is deleted or false if there was a problem deleting the folder.
	 */
	@Override
	public boolean deleteFolder(String folderPath, boolean recursive, String authentication, Session session)
	{
		File folder = new File(BASE_FOLDER, folderPath);
		
		if (recursive)
		{
			return deleteRecursive(folder, authentication, session);
		}
		else
		{
			return folder.delete();
		}
	}
	
	private boolean deleteRecursive(File file, String authentication, Session session)
	{
		if (file.isDirectory())
		{
			for (File f : file.listFiles())
			{
				if (!deleteRecursive(f, authentication, session))
				{
					return false;
				}
			}
		}
		return file.delete();
	}
	
	/**
	 * Gets a list of the names of the files and directories in the specified folder.
	 * @param folderPath  The path of the folder relative to the base directory.
	 * @param authentication The Authorization header from the request.
	 * @param session The existing Session object.
	 * @return a list of the names of the files and directories in the specified folder.
	 */
	@Override
	public List<String> getChildren(String folderPath, String authentication, Session session)
	{
		List<String> ret = new ArrayList<String>();
		File folder = new File(BASE_FOLDER, folderPath);
		File [] files = folder.listFiles(new FileFilter()
		{
			public boolean accept(File file)
			{
				return file.isDirectory() || file.getName().endsWith(".pdf");
			}
		});
		
		for (File f : files)
		{
			ret.add(f.getName());
		}
		return ret;
	
	}

	/**
	 * Validates the provider configuration.
	 * @return true if the provider configuration is valid.
	 */
	@Override
	public boolean isValidateConfiguration()
	{
		return true;
	}
	
	/**
	 * Creates a new session and returns the new session id.
	 * @param authentication The Authorization header from the request.
	 * @return The new Session.
	 */
	@Override
	public Session createSession(String authentication)
	{
		return new SampleFileSession(authentication, false);
	}

	/**
	 * Returns true if authorization is required.
	 * @return true if authorization is required.
	 */
	@Override
	public boolean isAuthorizationRequired()
	{
		return false;
	}
	
	private void copyStreams(InputStream instream, OutputStream outstream) throws IOException
	{
		int bytesRead;
		byte[] buffer = new byte[10240];

		while ((bytesRead = instream.read(buffer)) != -1)
		{
			outstream.write(buffer, 0, bytesRead);
		}
	}
}