package contentProvider;

import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.List;

import com.qoppa.spi.ContentProvider;
import com.qoppa.spi.Session;

import software.amazon.awssdk.auth.credentials.AwsBasicCredentials;
import software.amazon.awssdk.auth.credentials.StaticCredentialsProvider;
import software.amazon.awssdk.core.sync.RequestBody;
import software.amazon.awssdk.regions.Region;
import software.amazon.awssdk.services.s3.S3Client;
import software.amazon.awssdk.services.s3.model.DeleteObjectRequest;
import software.amazon.awssdk.services.s3.model.GetObjectRequest;
import software.amazon.awssdk.services.s3.model.HeadObjectRequest;
import software.amazon.awssdk.services.s3.model.HeadObjectResponse;
import software.amazon.awssdk.services.s3.model.ListObjectsRequest;
import software.amazon.awssdk.services.s3.model.ListObjectsResponse;
import software.amazon.awssdk.services.s3.model.NoSuchKeyException;
import software.amazon.awssdk.services.s3.model.PutObjectRequest;
import software.amazon.awssdk.services.s3.model.S3Object;

/**
 * This class is a sample that implements the ContentProvider interface that works with PDF Automation Server to
 * get and save content.
 * 
 * The class defines a "base folder", under which all files should be located.
 * 
 * @author Qoppa Software
 *
 */
public class SampleS3ContentProvider implements ContentProvider
{
	private final static String BUCKET_NAME = "bucket name";
	private final static String ACCESS_KEY_ID = "access key id";
	private final static String SECRET_ACCESS_KEY = "secret access key";

	/**
	 * Gets the human readable name of the provider.
	 * @return The human readable name of the provider
	 */
	@Override
	public String getProviderName()
	{
		return "Sample Amazon S3 Content Provider";
	}
	
	/**
	 * Create a new folder a the specified path.  This should also create any parent directories that don't yet exist.
	 * @param folderPath  The path to the new folder.  It is relative to the base directory.
	 * @param authentication The Authorization header from the request.
	 * @param session The existing Session object.
	 * @return True if the folder was create successfully or false if there was a problem creating the new folder or any of its ancestors.
	 */
	@Override
	public boolean createFolder(String folderPath, String authentication, Session session)
	{
		// There are no folders in Amazon S3
		return false;
	}
	
	/**
	 * Returns true if a file or folder exists at the specified path.
	 * @param path The path to the file or folder relative to the base directory.
	 * @param authentication The Authorization header from the request.
	 * @param session The existing Session object.
	 * @return True if the file or folder exists, false if it doesn't exist.
	 */
	@Override
	public boolean exists(String path, String authentication, Session session)
	{
		
		// Form the request
		HeadObjectRequest headRequest = HeadObjectRequest.builder()
				.bucket(((SampleS3Session)session).getBucketName())
				.key(path)
				.build();
		
		try
		{
			// Request header object
			((SampleS3Session)session).getClient().headObject(headRequest);
			return true;
		}
		catch(NoSuchKeyException nske)
		{
			return false;
		}
	}
	
	/**
	 * Gets the size in bytes of file at the specified path.
	 * @param path The path of the file relative to the base directory.
	 * @param authentication The Authorization header from the request.
	 * @param session The existing Session object.
	 * @return The size in bytes of the file at the specified path.
	 */
	@Override
	public long getFileSize(String path, String authentication, Session session)
	{
		// Form the request
		HeadObjectRequest headRequest = HeadObjectRequest.builder()
				.bucket(((SampleS3Session)session).getBucketName())
				.key(path)
				.build();
		
		try
		{
			// Request header object
			HeadObjectResponse resp = ((SampleS3Session)session).getClient().headObject(headRequest);
			return resp.contentLength().longValue();
		}
		catch(NoSuchKeyException nske)
		{
			return 0L;
		}
	}
	
	/**
	 * Gets the input stream of the file at the specified path.
	 * @param path The path of the file relative to the base directory.
	 * @param authentication The Authorization header from the request.
	 * @param session The existing Session object.
	 * @return The input stream of the file at the specified path.
	 * @throws IOException
	 */
	@Override
	public InputStream getInputStream(String path, String authentication, Session session) throws IOException
	{
		// Form the request
		GetObjectRequest getRequest = GetObjectRequest.builder()
				.bucket(((SampleS3Session)session).getBucketName())
				.key(path)
				.build();
		
		return ((SampleS3Session)session).getClient().getObject(getRequest);
	}

	/**
	 * Uploads the file to the specified path.
	 * 
	 * @param in The file's input stream.
	 * @param length The length of the input stream.  May be -1 if unknown.
	 * @param path The path of the file relative to the base directory.
	 * @param authentication The Authorization header from the request.
	 * @param session The existing Session object.
	 */
	@Override
	public void upload(InputStream instream, long length, String path, String authentication, Session session)
	{
		// Form the request
        PutObjectRequest objectRequest = PutObjectRequest.builder()
                .bucket(((SampleS3Session)session).getBucketName())
                .key(path)
                .build();

        // Upload the file
        ((SampleS3Session)session).getClient().putObject(objectRequest, RequestBody.fromInputStream(instream, 0));
	}
	
	/**
	 * Deletes the file at the specified path.
	 * 
	 * @param path The path of the file relative to the base directory.
	 * @param authentication The Authorization header from the request.
	 * @param session The existing Session object.
	 * @return True if the file was successfully deleted or false if there was a problem deleting the file.
	 */
	@Override
	public boolean deleteFile(String path, String authentication, Session session)
	{
		// Form the request
		DeleteObjectRequest delRequest = DeleteObjectRequest.builder()
				.bucket(((SampleS3Session)session).getBucketName())
				.key(path)
				.build();
		
		// Delete the object
		try
		{
			((SampleS3Session)session).getClient().deleteObject(delRequest);
			return true;
		}
		catch(Exception e)
		{
			return false;
		}
	}
	
	/**
	 * Deletes the folder at the specified path.  If recursive is set to false, the folder should only be deleted if it is empty.
	 * @param folderPath The path of the folder relative to the base directory.
	 * @param recursive Set to true if all sub-folders should be deleted.  
	 * @param authentication The Authorization header from the request.
	 * @param session The existing Session object.
	 * @return True if the folder is deleted or false if there was a problem deleting the folder.
	 */
	@Override
	public boolean deleteFolder(String folderPath, boolean recursive, String authentication, Session session)
	{
		//
		// There are no folders in S3 !
		//
		return false;
	}
	
	/**
	 * Gets a list of the names of the files and directories in the specified folder.
	 * @param folderPath  The path of the folder relative to the base directory.
	 * @param authentication The Authorization header from the request.
	 * @param session The existing Session object.
	 * @return a list of the names of the files and directories in the specified folder.
	 */
	@Override
	public List<String> getChildren(String folderPath, String authentication, Session session)
	{
		//
		// Always return the list of objects in the bucket, there are no folders
		//
		ListObjectsRequest request = ListObjectsRequest.builder()
				.bucket(((SampleS3Session)session).getBucketName())
				.build();
		
		ListObjectsResponse resp = ((SampleS3Session)session).getClient().listObjects(request);
		
		List<String> objList = new ArrayList<>();
		for (S3Object obj : resp.contents())
		{
			objList.add(obj.key());
		}
		
		return objList;
	}

	/**
	 * Validates the provider configuration.
	 * @return true if the provider configuration is valid.
	 */
	@Override
	public boolean isValidateConfiguration()
	{
		return true;
	}
	
	/**
	 * Creates a new session and returns the new session id.
	 * @param authentication The Authorization header from the request.
	 * @return The new Session.
	 */
	@Override
	public Session createSession(String authentication)
	{
		// Create S3 Client
		Region region = Region.US_EAST_1;
        S3Client s3Client = S3Client
        		.builder()
        		.region(region)
        		.credentialsProvider(StaticCredentialsProvider.create(AwsBasicCredentials.create(ACCESS_KEY_ID, SECRET_ACCESS_KEY)))
        		.build();

        return new SampleS3Session(s3Client, BUCKET_NAME);
	}

	/**
	 * Returns true if authorization is required.
	 * @return true if authorization is required.
	 */
	@Override
	public boolean isAuthorizationRequired()
	{
		return false;
	}
}
